"""Client library for encapsulating and managing the interaction with the
Calibration Constants Catalogue Web Application"""

__author__ = 'Luís Maia <luis.maia@xfel.eu>'
__version__ = '11.0.0'

from .calibration_client import CalibrationClient
